#ifndef _SWELLINGTON_MIFARE_DEF_H
#define _SWELLINGTON_MIFARE_DEF_H

#include <stdint.h>

#pragma once

namespace Swellington
{
    namespace Mifare
    {
        typedef struct
        {
            uint8_t size;
            uint8_t uidByte[10];
            uint8_t sak;
        } Uid;

        enum PCD_Register : uint8_t
        {
            CommandReg = 0x02,
            ComIEnReg = 0x04,
            DivIEnReg = 0x06,
            ComIrqReg = 0x08,
            DivIrqReg = 0x0A,
            ErrorReg = 0x0C,
            Status1Reg = 0x0E,
            Status2Reg = 0x10,
            FIFODataReg = 0x12,
            FIFOLevelReg = 0x14,
            WaterLevelReg = 0x16,
            ControlReg = 0x18,
            BitFramingReg = 0x1A,
            CollReg = 0x1C,

            ModeReg = 0x22,
            TxModeReg = 0x24,
            RxModeReg = 0x26,
            TxControlReg = 0x28,
            TxASKReg = 0x2A,
            TxSelReg = 0x2C,
            RxSelReg = 0x2E,
            RxThresholdReg = 0x30,
            DemodReg = 0x32,

            MfTxReg = 0x38,
            MfRxReg = 0x3A,

            SerialSpeedReg = 0x3E,
            CRCResultRegH = 0x42,
            CRCResultRegL = 0x44,
            ModWidthReg = 0x48,

            RFCfgReg = 0x4C,
            GsNReg = 0x4E,
            CWGsPReg = 0x50,
            ModGsPReg = 0x52,
            TModeReg = 0x54,
            TPrescalerReg = 0x56,
            TReloadRegH = 0x58,
            TReloadRegL = 0x5A,
            TCounterValueRegH = 0x5C,
            TCounterValueRegL = 0x5E,

            TestSel1Reg = 0x62,
            TestSel2Reg = 0x64,
            TestPinEnReg = 0x66,
            TestPinValueReg = 0x68,
            TestBusReg = 0x6A,
            AutoTestReg = 0x6C,
            VersionReg = 0x6E,
            AnalogTestReg = 0x70,
            TestDAC1Reg = 0x72,
            TestDAC2Reg = 0x74,
            TestADCReg = 0x76
        };

        enum PCD_Command : uint8_t
        {
            PCD_Idle = 0x00,
            PCD_Mem = 0x01,
            PCD_GenerateRandomID = 0x02,
            PCD_CalcCRC = 0x03,
            PCD_Transmit = 0x04,
            PCD_NoCmdChange = 0x07,
            PCD_Receive = 0x08,
            PCD_Transceive = 0x0C,
            PCD_MFAuthent = 0x0E,
            PCD_SoftReset = 0x0F
        };

        enum PCD_RxGain : uint8_t
        {
            RxGain_18dB = 0x00,
            RxGain_23dB = 0x10,
            RxGain_18dB_2 = 0x20,
            RxGain_23dB_2 = 0x30,
            RxGain_33dB = 0x40,
            RxGain_38dB = 0x50,
            RxGain_43dB = 0x60,
            RxGain_48dB = 0x70,
            RxGain_min = 0x00,
            RxGain_avg = 0x40,
            RxGain_max = 0x70
        };

        enum PICC_Command : uint8_t
        {
            PICC_CMD_REQA = 0x26,
            PICC_CMD_WUPA = 0x52,
            PICC_CMD_CT = 0x88,
            PICC_CMD_SEL_CL1 = 0x93,
            PICC_CMD_SEL_CL2 = 0x95,
            PICC_CMD_SEL_CL3 = 0x97,
            PICC_CMD_HLTA = 0x50,
            PICC_CMD_RATS = 0xE0,

            PICC_CMD_MF_AUTH_KEY_A = 0x60,
            PICC_CMD_MF_AUTH_KEY_B = 0x61,
            PICC_CMD_MF_READ = 0x30,
            PICC_CMD_MF_WRITE = 0xA0,
            PICC_CMD_MF_DECREMENT = 0xC0,
            PICC_CMD_MF_INCREMENT = 0xC1,
            PICC_CMD_MF_RESTORE = 0xC2,
            PICC_CMD_MF_TRANSFER = 0xB0,

            PICC_CMD_UL_WRITE = 0xA2
        };

        enum StatusCode : uint8_t
        {
            STATUS_OK,
            STATUS_ERROR,
            STATUS_COLLISION,
            STATUS_TIMEOUT,
            STATUS_NO_ROOM,
            STATUS_INTERNAL_ERROR,
            STATUS_INVALID,
            STATUS_CRC_WRONG,
            STATUS_MIFARE_NACK = 0xff
        };
    }

}

#endif